<?php
/** \file system/framework/cache.php
 * Система кэширования.
 */
/**
 * @project Astra.CMS
 * @link http://a-cms.ru/
 * @copyright 2011 "Астра Вебтехнологии"
 * @author Vitaly Hohlov <admin@a-cms.ru>
 * @package AFramework
 */
/**************************************************************************/

/**
 * Общая система кэширования.
 */

class A_Cache
{
  private static $instance;

/**
 * Объект системы кэширования БД.
 */

  public $db;

/**
 * Объект системы кэширования страниц.
 */

  public $page;

/**
 * Объект системы кэширования блоков.
 */

  public $block;

/**
 * Конструктор.
 */

  function __construct()
  {
    switch(A::$OPTIONS['caching'])
	{ case 1:
	    require_once("system/framework/fcache.php");
	    $this->db = new A_Cache_DB_File;
	    if(A_MODE==A_MODE_FRONT)
        { $this->page = new A_Cache_Page_File;
	      $this->block = new A_Cache_Block_File;
	    }
		break;
	  case 2:
	    if(extension_loaded('memcache'))
		{ require_once("system/framework/mcache.php");
		  $this->db = new A_Cache_DB_Mem;
		  if(A_MODE==A_MODE_FRONT)
		  { $this->page = new A_Cache_Page_Mem;
		    $this->block = new A_Cache_Block_Mem;
		  }
		}
		break;
	}
  }

/**
 * Возвращает объект системы кэширования.
 * @return object Объект системы кэширования.
 */

  function getInstance()
  {
    if(!self::$instance)
    self::$instance = new A_Cache;
    return self::$instance;
  }

/**
 * Сбрасывает весь кэш связанный с таблицей БД.
 * @param string $table Таблица БД.
 */

  function resetTable($table)
  {
    if($this->db)
    $this->db->resetTable($table);
  }

/**
 * Сбрасывает весь кэш связанный с разделом.
 * @param string $section Полный строковой идентификатор раздела.
 */

  function resetSection($section)
  {
    if($this->db)
    $this->db->resetSection($section);
  }

/**
 * Сбрасывает весь кэш связанный с сайтом.
 * @param string $domain Идентификатор сайта.
 */

  function resetDomain($domain)
  {
    if($this->db)
    switch(A::$OPTIONS['caching'])
    { case 1:
        clearDir("cache/db/".$domain);
        clearDir("cache/pages/".$domain);
        clearDir("cache/vblocks/".$domain);
        break;
      case 2:
        $this->db->resetDomain($domain);
		break;
    }
  }

/**
 * Сбрасывает весь кэш.
 */

  function resetAll()
  {
    if($this->db)
	switch(A::$OPTIONS['caching'])
    { case 1:
        clearDir("cache/db");
        clearDir("cache/pages");
        clearDir("cache/vblocks");
        break;
      case 2:
        A_MemCache::getInstance()->flush();
		break;
    }
  }
}