<?php
/** \file system/framework/db.php
 * Работа с БД MySQL.
 */
/**
 * @project Astra.CMS
 * @link http://a-cms.ru/
 * @copyright 2009 "Астра Вебтехнологии"
 * @author Vitaly Hohlov <admin@a-cms.ru>
 * @package AFramework
 */
/**************************************************************************/

/**
 * Исключение при работе с БД.
 */

class A_DBException extends LogicException {}

/**
 * Исключение при подключении к БД.
 */

class A_DBExceptionConnect extends A_DBException {}

/**
 * Исключение при запросе к БД.
 */

class A_DBExceptionQuery extends A_DBException
{
  private $query, $errno, $error;

  function __construct($query,$errno,$error)
  {
     $this->query = $query;
     $this->errno = $errno;
     $this->error = $error;
  }

  function query() { return $this->query; }
  function errno() { return $this->errno; }
  function error() { return $this->error; }

  public function __toString()
  {
    return 'DB Error: '.$this->errno.' "'.$this->error().'" SQL="'.$this->query.'"';
  }
}

/**
 * Исключение при формировании запроса.
 */

class A_DBExceptionData extends A_DBException
{
  protected $message = 'DB Data Exception';
}

/**
 * Исключение при формировании запроса (Лишние параметры).
 */

class A_DBExceptionDataMuch extends A_DBExceptionData
{
  protected $message = 'It is too much data';
}

/**
 * Исключение при формировании запроса (Не хватает параметров).
 */

class A_DBExceptionDataNotEnough extends A_DBExceptionData
{
  protected $message = 'It is not enough data';
}

/**
 * Класс реализует основные функции для работы с БД MySQL.
 * Содержит встроенную интеграцию с системой кэширования.
 * Автоматически поддерживает работу с дополнительными базами данных.
 * Поддерживаются параметризированные запросы.
 */

class A_DB
{
  private static $instance;

  private static $index=-1;

  private $mqph;

  private $connections;

  private $connection;

/**
 * Стек всех результатов запросов.
 */

  public $results=array();

/**
 * Объект кэширования БД.
 */

  public $cache;

/**
 * Флаг состояния кэширования (true/false).
 */

  public $caching;

/**
 * Количество подключений к БД.
 */

  public $ccounter=0;

/**
 * Количество запросов к БД.
 */

  public $qcounter=0;

/**
 * Возвращает объект для работы с БД.
 * @return object Объект для работы с БД.
 */

  function getInstance()
  {
    if(!self::$instance)
    self::$instance = new A_DB;
    return self::$instance;
  }

/**
 * Подключение к БД.
 *
 * @param string Идентификатор сайта.
 */

  function connect($domain='')
  { static $bases=array();
	if(!empty($domain))
	{ if(!isset($bases[$domain]))
	  { $base=$this->getRow("SELECT host,name,user,password FROM _bases WHERE id=(SELECT idbase FROM _domains WHERE name=?)",$domain);
	    $bases[$domain]=$base?$base:false;
	  }
	  else
	  $base=$bases[$domain];
	}
	if(empty($base))
	$base=$GLOBALS['A_DBCONFIG'];
	if(!isset($this->connections[$base['name']]))
	{ $this->connections[$base['name']] = new mysqli($base['host'],$base['user'],$base['password'],$base['name']);
	  if(mysqli_connect_errno())
	  throw new A_DBExceptionConnect(mysqli_connect_error(),mysqli_connect_errno());
	  $this->connections[$base['name']]->query("SET NAMES 'utf8'");
	  $this->ccounter++;
	}
	$this->connection=$this->connections[$base['name']];
  }

  function real_escape_string($string)
  {
	if(!$this->connection)
	$this->connect();
	return $this->connection->real_escape_string($string);
  }

/**
 * Выполнение SQL запроса.
 *
 * @param string $sql Строка запроса.
 * @param array $params=array() Параметры запроса.
 * @param string table='' Таблица БД (для запросов на изменение).
 * @return true/false или MySQLi_RESULT
 */

  function execute($sql,$params=array(),$table='')
  {
    $sql=$params?$this->makeSQL($sql,$params):$sql;
	if(preg_match("/^[\s]*(INSERT IGNORE INTO|UPDATE|DELETE FROM|ALTER TABLE|DROP TABLE|TRUNCATE)[\s]*`?([a-zA-Z0-9_]+)`?/i",$sql,$matches))
	{ if(!preg_match("/^".A::$DOMAIN."_/i",$matches[2]))
	  { if(!(A_MODE==1 && A::$AUTH->isSuperAdmin()))
	    return null;
	  }
	  if($this->caching)
	  A::$CACHE->resetTable($matches[2]);
	  $result=$this->_execute($sql,$matches[2]);
	}
	else
	$result=$this->_execute($sql,$table);
	return $result;
  }

/**
 * Выполнение SQL запроса.
 * Не поддерживает параметры и работает в обход системы кэширования.
 *
 * @param string $sql Строка запроса.
 * @param string table='' Таблица БД (для запросов на изменение).
 * @return true/false или MySQLi_RESULT
 */

  function _execute($sql,$table='')
  {
    if(!empty($GLOBALS['A_DBCONFIG']['sqlreplace']))
    foreach($GLOBALS['A_DBCONFIG']['sqlreplace'] as $in=>$to)
	{ $sql=str_replace($in,$to,$sql);
	  $table=str_replace($in,$to,$table);
	}
	if(empty($table) && $tables=$this->getUseTables($sql))
	$table=array_shift($tables);
	if(preg_match("/^`?([a-zA-Z0-9]+)_/i",$table,$matches))
	$this->connect($matches[1]);
	else
	$this->connect();
	return $this->__execute($sql);
  }

/**
 * Выполнение SQL запроса.
 * Не поддерживает параметры, а так же работает в обход системы кэширования и дополнительных БД.
 *
 * @param string $sql Строка запроса.
 * @return mixed true/false или MySQLi_RESULT
 */

  function __execute($sql)
  {
	$result=$this->connection->query($sql,MYSQLI_STORE_RESULT);
    if($this->connection->errno)
	throw new A_DBExceptionQuery($sql,$this->connection->errno,$this->connection->error);
	$this->qcounter++;
	return $result;
  }

/**
 * Формирование SQL запроса.
 *
 * @param string $pattern Шаблон запроса.
 * @param array $params Параметры запроса.
 * @return string строка SQL запроса.
 */

  function makeSQL($pattern,$params)
  {
	$this->mqph = !is_array($params)?array($params):$params;
    $sql = preg_replace_callback('/\?([int?casv]?[ina]?);?/u',array($this,'_makeSQL'),$pattern);
    if(count($this->mqph)>0)
	throw new A_DBExceptionDataMuch();
    return $sql;
  }

  private function _makeSQL($ph)
  {
    if($ph[1] == '?')
	return '?';
    if(count($this->mqph)==0)
	throw new A_DBExceptionDataNotEnough();
    $el=array_shift($this->mqph);
    switch($ph[1])
	{ case ('i'): return (integer)$el;
      case ('t'): return '`'.$el.'`';
      case ('c'): return '`'.$el.'`';
      case ('n'): return is_null($el)?'NULL':('"'.$this->real_escape_string($el).'"');
      case ('ni'):
      case ('in'): return is_null($el)?'NULL':(integer)$el;
      case ('a'):
        foreach($el as &$e)
		$e='"'.$this->real_escape_string($e).'"';
        return implode(',',$el);
      case ('ai'):
      case ('ia'):
        foreach($el as &$e)
		$e=(integer)$e;
        return implode(',',$el);
      case ('s'):
        $set=array();
        foreach($el as $k=>$v)
		{ if($v !== null)
		  $set[]='`'.$k.'`="'.$this->real_escape_string($v).'"';
          else
		  $set[]='`'.$k.'`=NULL';
        }
        return implode(',',$set);
      case ('v'):
       	$valueses=array();
        foreach($el as $v)
		{ $values=array();
          foreach($v as $d)
		  { if($d!==null)
		    $values[]='"'.$this->real_escape_string($d).'"';
            else
			$values[]='NULL';
          }
          $valueses[]='('.implode(',',$values).')';
        }
        return implode(',',$valueses);
    }
    return '"'.$this->real_escape_string($el).'"';
  }

/**
 * Определение всех таблиц БД участвующих в запросе.
 *
 * @param string $sql Строка SQL запроса
 * @return array Список таблиц.
 */

  function getUseTables($sql)
  {
    $c=0;
    $tables=array();
    $sql='('.$sql.')';
    while(preg_match('/ \( (?:[\x20\r\n\t]*) SELECT (?![a-z\d_]) /sxiu',$sql))
    { $sql = preg_replace('/\((?>(?>[^()](?<![^a-z\d_]SELECT[^a-z\d_]))+|(?R))*\)/sxiu','{_BRACKETS_}',$sql);
      $j = 0;
      while(preg_match('/ \((?>[\x20\r\n\t]*) SELECT (?![a-z\d_])[^()]+?(?<![a-z\d_]) FROM (?![a-z\d_])([^()]+?)(?:(?<![a-z\d_])(?>WHERE|GROUP|HAVING|ORDER|LIMIT|PROCEDURE) (?![a-z\d_])[^()]+?)?\)/sxiu',$sql,$m))
      { $j++;
        foreach(preg_split('/(?>,|(?<![a-z\d_])JOIN(?![a-z\d_]))/siu',$m[1]) as $i=>$v)
        { $t=preg_split('/[\x20\r\n\t]+/siu', trim($v), 2);
          if($t[0]!=='{_SELECT_}') $tables[]=str_replace('`','',$t[0]);
        }
		$sql=str_replace($m[0],'{_SELECT_}',$sql);
        if(++$c>=99)
        return false;
      }
      if($j==0)
      return false;
    }
    return array_unique($tables);
  }

/**
 * Проверка на существование записи.
 *
 * @param string $sql Строка SQL запроса
 * @param array $params=array() Параметры запроса.
 * @return boolean true если хоть одна запись существует.
 */

  function existsRow($sql,$params=array())
  {
	$sql=$params?$this->makeSQL($sql,$params):$sql;
	if($this->caching)
	{ if($data=$this->cache->get($sql,'bool'))
	  return $data['data'];
	  else
	  { $data=$this->_execute($sql)->num_rows>0;
	    return $this->cache->set($sql,$data,'bool');
	  }
	}
	else
	return $this->_execute($sql)->num_rows>0;
  }

/**
 * Проверка на существование записи.
 * Не поддерживает параметры и работает в обход системы кэширования.
 *
 * @param string $sql Строка SQL запроса
 * @return boolean true если хоть одна запись существует.
 */

  function _existsRow($sql)
  {
	return $this->_execute($sql)->num_rows>0;
  }

/**
 * Возвращает первую запись по результату SQL запроса.
 *
 * @param string $sql Строка SQL запроса
 * @param array $params=array() Параметры запроса.
 * @return array() Ассоцированный массив с данными записи.
 */

  function getRow($sql,$params=array())
  {
	$sql=$params?$this->makeSQL($sql,$params):$sql;
	if($this->caching)
	{ if($data=$this->cache->get($sql,'row'))
	  return $data['data'];
	  else
	  { $data=$this->_execute($sql)->fetch_assoc();
	    return $this->cache->set($sql,$data,'row');
	  }
	}
	else
	return $this->_execute($sql)->fetch_assoc();
  }

/**
 * Возвращает первую запись по результату SQL запроса.
 * Не поддерживает параметры и работает в обход системы кэширования.
 *
 * @param string $sql Строка SQL запроса
 * @return array() Ассоцированный массив с данными записи.
 */

  function _getRow($sql)
  {
	return $this->_execute($sql)->fetch_assoc();
  }

/**
 * Извлекает запись из таблицы БД по значению id.
 *
 * @param integer $id Уникальный идентификатор записи в таблице.
 * @param string $table Таблица БД.
 * @return array() Ассоцированный массив с данными записи.
 */

  function getRowById($id,$table)
  {
	$sql="SELECT * FROM `$table` WHERE id=".(integer)$id;
	if($this->caching)
	{ if($data=$this->cache->get($sql,'row'))
	  return $data['data'];
	  else
	  { $data=$this->_execute($sql,$table)->fetch_assoc();
	    return $this->cache->set($sql,$data,'row');
	  }
	}
	else
	return $this->_execute($sql,$table)->fetch_assoc();
  }

/**
 * Извлекает запись из таблицы БД по значению id.
 * Работает в обход системы кэширования.
 *
 * @param integer $id Уникальный идентификатор записи в таблице.
 * @param string $table Таблица БД.
 * @return array() Ассоцированный массив с данными записи.
 */

  function _getRowById($id,$table)
  {
	return $this->_execute("SELECT * FROM `$table` WHERE id=".(integer)$id,$table)->fetch_assoc();
  }

/**
 * Возвращает значение первого поля в первой записи по результату SQL запроса.
 *
 * @param string $sql Строка SQL запроса
 * @param array $params=array() Параметры запроса.
 * @return mixed Значение.
 */

  function getOne($sql,$params=array())
  {
    $sql=$params?$this->makeSQL($sql,$params):$sql;
    if($this->caching)
	{ if($data=$this->cache->get($sql,'one'))
	  return $data['data'];
	  else
	  { $result=$this->_execute($sql)->fetch_row();
	    $data=$result?array_shift($result):false;
	    return $this->cache->set($sql,$data,'one');
	  }
	}
	else
	{ $result=$this->_execute($sql)->fetch_row();
      return $result?array_shift($result):false;
	}
  }

/**
 * Возвращает значение первого поля в первой записи по результату SQL запроса.
 * Работает в обход системы кэширования.
 *
 * @param string $sql Строка SQL запроса*
 * @return mixed Значение.
 */

  function _getOne($sql)
  {
    $result=$this->_execute($sql)->fetch_row();
    return $result?array_shift($result):false;
  }

/**
 * Возвращает асоциированный массив, ключом в котором является значение первого поля в записях по результату SQL запроса.
 * Если в записях только два поля, то второе является значением элемента массива, если больше, то значением элемента становится массив из оставшихся полей.
 *
 * @param string $sql Строка SQL запроса
 * @param array $params=array() Параметры запроса.
 * @return array Ассоциированный массив.
 */

  function getAssoc($sql,$params=array())
  {
    $sql=$params?$this->makeSQL($sql,$params):$sql;
	if($this->caching)
	{ if($data=$this->cache->get($sql,'assoc'))
	  return $data['data'];
	  else
	  { $data=$this->_getAssoc($sql);
	    $data=$this->cache->set($sql,$data,'assoc');
	    return $data?$data:array();
	  }
	}
	else
	return $this->_getAssoc($sql);
  }

/**
 * Возвращает асоциированный массив, ключом в котором является значение первого поля в записях по результату SQL запроса.
 * Если в записях только два поля, то второе является значением элемента массива, если больше, то значением элемента становится массив из оставшихся полей.
 * Не поддерживает параметры и работает в обход системы кэширования.
 *
 * @param string $sql Строка SQL запроса
 * @return array Ассоциированный массив.
 */

  function _getAssoc($sql)
  {
    $data=array();
	$result=$this->_execute($sql);
	if($result->field_count==2)
	{ while($row=$result->fetch_row())
	  $data[$row[0]]=$row[1];
	}
	elseif($result->field_count>2)
	{ while($row=$result->fetch_assoc())
	  $data[array_shift($row)]=$row;
	}
	$result->free();
    return $data;
  }

/**
 * Возвращает массив, элементами в которого являются значения первого поля в записях по результату SQL запроса.
 *
 * @param string $sql Строка SQL запроса
 * @param array $params=array() Параметры запроса.
 * @return array Ассоциированный массив.
 */

  function getCol($sql,$params=array())
  {
    $sql=$params?$this->makeSQL($sql,$params):$sql;
	if($this->caching)
	{ if($data=$this->cache->get($sql,'col'))
	  return $data['data'];
	  else
	  { $data=$this->_getCol($sql);
	    return $this->cache->set($sql,$data,'col');
	  }
	}
	else
	return $this->_getCol($sql);
  }

/**
 * Возвращает массив, элементами в которого являются значения первого поля в записях по результату SQL запроса.
 * Не поддерживает параметры и работает в обход системы кэширования.
 *
 * @param string $sql Строка SQL запроса
 * @param array $params=array() Параметры запроса.
 * @return array Ассоциированный массив.
 */

  function _getCol($sql)
  {
    $data=array();
	$result=$this->_execute($sql);
	if($result->field_count>0)
	while($row=$result->fetch_row())
	$data[]=$row[0];
	$result->free();
    return $data;
  }

/**
 * Возвращает массив всех записей по результату SQL запроса.
 *
 * @param string $sql Строка SQL запроса
 * @param array $params=array() Параметры запроса.
 * @return array Массив записей, каждая из которых - асоциированный массив значений полей.
 */

  function getAll($sql,$params=array())
  {
    $sql=$params?$this->makeSQL($sql,$params):$sql;
    if($this->caching)
	{ if($data=$this->cache->get($sql))
	  return $data['data'];
	  else
	  { $data=$this->_getAll($sql);
	    return $this->cache->set($sql,$data);
	  }
	}
	else
	return $this->_getAll($sql);
  }

/**
 * Возвращает массив всех записей по результату SQL запроса.
 * Не поддерживает параметры и работает в обход системы кэширования.
 *
 * @param string $sql Строка SQL запроса
 * @param array $params=array() Параметры запроса.
 * @return array Массив записей, каждая из которых - асоциированный массив значений полей.
 */

  function _getAll($sql)
  {
    $data=array();
	$result=$this->_execute($sql);
	if($result->field_count>0)
	while($row=$result->fetch_assoc())
	$data[]=$row;
	$result->free();
    return $data;
  }

/**
 * Возвращает количество найденных записей по результату SQL запроса.
 *
 * @param string $table Таблица БД.
 * @param string $where Условие выборки.
 * @param array $params=array() Параметры запроса.
 * @return integer Количество найденных записей.
 */

  function getCount($table,$where='',$params=array())
  {
    $sql="SELECT COUNT(*) FROM `$table`".(!empty($where)?" WHERE $where":"");
	$params=!is_array($params)?array($params):$params;
	$sql=$params?$this->makeSQL($sql,$params):$sql;
	if($this->caching)
	{ if($data=$this->cache->get($sql,'one'))
	  return (integer)$data['data'];
	  else
	  { $data=array_shift($this->_execute($sql,$table)->fetch_row());
	    return (integer)$this->cache->set($sql,$data,'one');
	  }
	}
	else
    return (integer)array_shift($this->_execute($sql,$table)->fetch_row());
  }

/**
 * Возвращает количество найденных записей по результату SQL запроса.
 * Не поддерживает параметры и работает в обход системы кэширования.
 *
 * @param string $table Таблица БД.
 * @param string $where Условие выборки.
 * @param array $params=array() Параметры запроса.
 * @return integer Количество найденных записей.
 */

  function _getCount($table,$where='')
  {
    $sql="SELECT COUNT(*) FROM `$table`".(!empty($where)?" WHERE $where":"");
    return (integer)array_shift($this->_execute($sql,$table)->fetch_row());
  }

/**
 * Выполняет SQL запрос и сохраняет результат в стеке для последующего извлечения с помощью fetchRow().
 *
 * @param string $sql Строка SQL запроса
 * @param array $params=array() Параметры запроса.
 * @return integer Количество записей в результате.
 */

  function query($sql,$params=array())
  {
    $sql=$params?$this->makeSQL($sql,$params):$sql;
    if($this->caching)
	{ if($data=$this->cache->get($sql))
	  { $this->results[++self::$index]=$data['data'];
	    return count($data['data']);
	  }
	  else
	  { $this->results[++self::$index]=$this->_getAll($sql);
		return count($this->cache->set($sql,$this->results[self::$index]));
	  }
	}
	else
    { $this->results[++self::$index]=$this->_getAll($sql);
	  return count($this->results[self::$index]);
	}
  }

/**
 * Выполняет SQL запрос и сохраняет результат в стеке для последующего извлечения с помощью fetchRow().
 * Не поддерживает параметры и работает в обход системы кэширования.
 *
 * @param string $sql Строка SQL запроса
 * @param array $params=array() Параметры запроса.
 * @return integer Количество записей в результате.
 */

  function _query($sql)
  {
    $this->results[++self::$index]=$this->_getAll($sql);
	return count($this->results[self::$index]);
  }

/**
 * Выполняет SQL запрос с заданным лимитом выборки и сохраняет результат в стеке для последующего извлечения с помощью fetchRow().
 *
 * @param string $sql Строка SQL запроса
 * @param integer $b Номер первой записи в выборке.
 * @param integer $c Количество записей.
 * @param array $params=array() Параметры запроса.
 * @return integer Общее количество найденных записей.
 */

  function queryLimit($sql,$b,$c,$params=array())
  {
    $b=(integer)$b;
    $c=(integer)$c;
	$sql=preg_replace("/^[\s]*SELECT/i","SELECT SQL_CALC_FOUND_ROWS",$sql)." LIMIT $b,$c";
	$sql=$params?$this->makeSQL($sql,$params):$sql;
    if($this->caching)
	{ if($data=$this->cache->get($sql))
	  { $this->results[++self::$index]=isset($data['data'][0])?$data['data'][0]:$data['data'];
	    return !empty($data['data'][1])?$data['data'][1]:(!empty($data['data'][0])?count($data['data'][0]):0);
	  }
	  else
	  { $this->results[++self::$index]=$this->_getAll($sql);
		$result=$this->connection->query("SELECT FOUND_ROWS() as __total_rows")->fetch_row();
		$this->cache->set($sql,array($this->results[self::$index],$found=$result?array_shift($result):0));
		return $found;
	  }
	}
	else
	{ $this->results[++self::$index]=$this->_getAll($sql);
	  $result=$this->connection->query("SELECT FOUND_ROWS() as __total_rows")->fetch_row();
	  return $result?array_shift($result):0;
	}
  }

/**
 * Выполняет SQL запрос с заданным лимитом выборки и сохраняет результат в стеке для последующего извлечения с помощью fetchRow().
 * Не поддерживает параметры и работает в обход системы кэширования.
 *
 * @param string $sql Строка SQL запроса
 * @param integer $b Номер первой записи в выборке.
 * @param integer $c Количество записей.
 * @param array $params=array() Параметры запроса.
 * @return integer Общее количество найденных записей.
 */

  function _queryLimit($sql,$b,$c)
  {
    $b=(integer)$b;
    $c=(integer)$c;
    $sql=preg_replace("/^[\s]*SELECT/i","SELECT SQL_CALC_FOUND_ROWS",$sql)." LIMIT $b,$c";
    $this->results[++self::$index]=$this->_getAll($sql);
	$result=$this->connection->query("SELECT FOUND_ROWS() as __total_rows")->fetch_row();
	return $result?array_shift($result):0;
  }

/**
 * Извлечение очередной записи из последнего результата в стеке.
 *
 * @return array Ассоциированный массив или false.
 */

  function fetchRow()
  {
	return !empty($this->results[self::$index])?array_shift($this->results[self::$index]):false;
  }

/**
 * Количество записей у последнего результата в стеке.
 *
 * @return array Ассоциированный массив.
 */

  function numRows()
  {
	return !empty($this->results[self::$index])?count($this->results[self::$index]):0;
  }

/**
 * Удаление последнего результата из стека.
 */

  function free()
  {
    unset($this->results[self::$index]);
	self::$index--;
  }

/**
 * Добавление новой записи в таблицу БД.
 *
 * @param string $table Таблица БД.
 * @param array $data Асоциированный массив со значениями полей.
 * @return integer Уникальный идентификатор новой записи или false в случае неудачи.
 */

  function Insert($table,$data)
  {
	if(empty($data)) return false;
	$fields=array_keys($data);
	$values=array_values($data);
	$sql="INSERT IGNORE INTO `$table` (".implode(",",array_fill(0,count($data),'?c')).") VALUES (".implode(",",array_fill(0,count($data),'?')).")";
	$this->execute($sql,array_merge($fields,$values),$table);
	if($this->connection->affected_rows>0)
	{ A::$OBSERVER->Event('InsertTable',$table,$this->connection->insert_id);
	  return $this->connection->insert_id;
	}
	else
	return false;
  }

/**
 * Добавление новой записи в таблицу БД.
 * Работает в обход системы кэширования.
 *
 * @param string $table Таблица БД.
 * @param array $data Асоциированный массив со значениями полей.
 * @return integer Уникальный идентификатор новой записи или false в случае неудачи.
 */

  function _Insert($table,$data)
  {
	if(empty($data)) return false;
	$fields=array_keys($data);
	$values=array_values($data);
	$sql="INSERT IGNORE INTO `$table` (".implode(",",array_fill(0,count($data),'?c')).") VALUES (".implode(",",array_fill(0,count($data),'?')).")";
	$sql=$this->makeSQL($sql,array_merge($fields,$values));
	$this->_execute($sql,$table);
	if($this->connection->affected_rows>0)
	{ A::$OBSERVER->Event('InsertTable',$table,$this->connection->insert_id);
	  return $this->connection->insert_id;
	}
	else
	return false;
  }

/**
 * Добавление новой записи в таблицу БД.
 * Работает в обход системы кэширования и системы дополнительных БД.
 *
 * @param string $table Таблица БД.
 * @param array $data Асоциированный массив со значениями полей.
 * @return integer Уникальный идентификатор новой записи или false в случае неудачи.
 */

  function __Insert($table,$data)
  {
	if(empty($data)) return false;
	$fields=array_keys($data);
	$values=array_values($data);
	$sql="INSERT IGNORE INTO `$table` (".implode(",",array_fill(0,count($data),'?c')).") VALUES (".implode(",",array_fill(0,count($data),'?')).")";
	$sql=$this->makeSQL($sql,array_merge($fields,$values));
	$this->__execute($sql);
	if($this->connection->affected_rows>0)
	return $this->connection->insert_id;
	else
	return false;
  }

/**
 * Обновление записей в таблице БД.
 *
 * @param string $table Таблица БД.
 * @param array $data Асоциированный массив со значениями полей.
 * @param array $where Условие в SQL запросе.
 * @param array $params=array() Параметры запроса.
 * @return integer Количество измененных записей.
 */

  function Update($table,$data,$where="",$params=array())
  {
    if(empty($data)) return false;
    $rparams=array();
	foreach($data as $field=>$value)
	{ $rparams[]=$field;
	  $rparams[]=$value;
	}
	$sql="UPDATE `$table` SET ".implode(",",array_fill(0,count($data),'?c=?'));
	if(!empty($where))
	$sql.=" WHERE $where";
	$this->execute($sql,array_merge($rparams,!is_array($params)?array($params):$params),$table);
	A::$OBSERVER->Event('UpdateTable',$table,$data);
	return $this->connection->affected_rows;
  }

/**
 * Обновление записей в таблице БД.
 * Не поддерживает параметры и работает в обход системы кэширования.
 *
 * @param string $table Таблица БД.
 * @param array $data Асоциированный массив со значениями полей.
 * @param array $where Условие в SQL запросе.
 * @param array $params=array() Параметры запроса.
 * @return integer Количество измененных записей.
 */

  function _Update($table,$data,$where="")
  {
    if(empty($data)) return false;
    $rparams=array();
	foreach($data as $field=>$value)
	{ $rparams[]=$field;
	  $rparams[]=$value;
	}
	$sql="UPDATE `$table` SET ".implode(",",array_fill(0,count($data),'?c=?'));
	if(!empty($where))
	$sql.=" WHERE $where";
	$sql=$this->makeSQL($sql,$rparams);
	$this->_execute($sql,$table);
	A::$OBSERVER->Event('UpdateTable',$table,$data);
	return $this->connection->affected_rows;
  }

/**
 * Обновление записей в таблице БД.
 * Не поддерживает параметры и работает в обход систем кэширования и дополнительных БД.
 *
 * @param string $table Таблица БД.
 * @param array $data Асоциированный массив со значениями полей.
 * @param array $where Условие в SQL запросе.
 * @param array $params=array() Параметры запроса.
 * @return integer Количество измененных записей.
 */

  function __Update($table,$data,$where="")
  {
    if(empty($data)) return false;
    $rparams=array();
	foreach($data as $field=>$value)
	{ $rparams[]=$field;
	  $rparams[]=$value;
	}
	$sql="UPDATE `$table` SET ".implode(",",array_fill(0,count($data),'?c=?'));
	if(!empty($where))
	$sql.=" WHERE $where";
	$sql=$this->makeSQL($sql,$rparams);
	$this->__execute($sql);
	return $this->connection->affected_rows;
  }

/**
 * Обновление записей в таблице БД, если обновлять нечего - добавляется новая запись.
 *
 * @param string $table Таблица БД.
 * @param array $data Асоциированный массив со значениями полей.
 * @param array $where Условие в SQL запросе.
 * @param array $params=array() Параметры запроса.
 * @return integer Уникальный идентификатор новой записи или 0 в случае обновления.
 */

  function Replace($table,$data,$where="",$params=array())
  {
    if(empty($data)) return false;
	$this->Update($table,$data,$where,$params);
	if($this->connection->affected_rows==0 && !isset($data['id']))
	return $this->Insert($table,$data);
	else
	return 0;
  }

/**
 * Обновление записей в таблице БД, если обновлять нечего - добавляется новая запись.
 * Не поддерживает параметры и работает в обход системы кэширования.
 *
 * @param string $table Таблица БД.
 * @param array $data Асоциированный массив со значениями полей.
 * @param array $where Условие в SQL запросе.
 * @param array $params=array() Параметры запроса.
 * @return integer Уникальный идентификатор новой записи или 0 в случае обновления.
 */

  function _Replace($table,$data,$where="",$params=array())
  {
    if(empty($data)) return false;
	$this->_Update($table,$data,$where,$params);
	if($this->connection->affected_rows==0 && !isset($data['id']))
	return $this->_Insert($table,$data);
	else
	return 0;
  }

/**
 * Обновление записей в таблице БД, если обновлять нечего - добавляется новая запись.
 * Не поддерживает параметры и работает в обход систем кэширования и дополнительных БД.
 *
 * @param string $table Таблица БД.
 * @param array $data Асоциированный массив со значениями полей.
 * @param array $where Условие в SQL запросе.
 * @param array $params=array() Параметры запроса.
 * @return integer Уникальный идентификатор новой записи или 0 в случае обновления.
 */

  function __Replace($table,$data,$where="",$params=array())
  {
    if(empty($data)) return false;
	$this->__Update($table,$data,$where,$params);
	if($this->connection->affected_rows==0 && !isset($data['id']))
	return $this->__Insert($table,$data);
	else
	return 0;
  }

/**
 * Удаление записей из таблицы БД.
 *
 * @param string $table Таблица БД.
 * @param array $where Условие в SQL запросе.
 * @param array $params=array() Параметры запроса.
 * @return integer Количество удаленных записей.
 */

  function Delete($table,$where="",$params=array())
  {
    $sql="DELETE FROM `$table`";
	if(!empty($where))
	$sql.=" WHERE $where";
    $this->execute($sql,!is_array($params)?array($params):$params,$table);
    A::$OBSERVER->Event('DeleteTable',$table);
	return $this->connection->affected_rows;
  }

/**
 * Удаление записей из таблицы БД.
 * Не поддерживает параметры и работает в обход системы кэширования.
 *
 * @param string $table Таблица БД.
 * @param array $where Условие в SQL запросе.
 * @param array $params=array() Параметры запроса.
 * @return integer Количество удаленных записей.
 */

  function _Delete($table,$where="")
  {
    $sql="DELETE FROM `$table`";
	if(!empty($where))
	$sql.=" WHERE $where";
    $this->_execute($sql,$table);
    A::$OBSERVER->Event('DeleteTable',$table);
	return $this->connection->affected_rows;
  }

/**
 * Удаление записей из таблицы БД.
 * Не поддерживает параметры и работает в обход систем кэширования и дополнительных БД.
 *
 * @param string $table Таблица БД.
 * @param array $where Условие в SQL запросе.
 * @param array $params=array() Параметры запроса.
 * @return integer Количество удаленных записей.
 */

  function __Delete($table,$where="")
  {
    $sql="DELETE FROM `$table`";
	if(!empty($where))
	$sql.=" WHERE $where";
    $this->__execute($sql);
	return $this->connection->affected_rows;
  }

/**
 * Возвращает количество записей, к которым был применен последний запрос.
 *
 * @return integer Количество записей, к которым был применен последний запрос.
 */

  function affectedRows()
  {
    return $this->connection->affected_rows;
  }

/**
 * Возвращает список таблиц БД принадлежащих сайту.
 *
 * @param string $domain='' Идентификатор сайта, если не указано то текущий.
 * @return array Список таблиц БД принадлежащих сайту.
 */

  function getTables($domain='')
  {
    $domain=empty($domain)?A::$DOMAIN:$domain;
    $this->connect($domain);
	$data=array();
	$result=$this->connection->query($sql="SHOW TABLES");
	if($this->connection->errno)
	throw new A_DBExceptionQuery($sql,$this->connection->errno,$this->connection->error);
	if($result->field_count>0)
	while($row=$result->fetch_row())
	$data[]=$row[0];
	$result->free();
    return $data;
  }

/**
 * Возвращает список полей в таблице БД.
 *
 * @param string $table Таблица БД.
 * @return array Список список полей в заданной таблице.
 */

  function getFields($table,$domain='')
  {
    $domain=empty($domain)?A::$DOMAIN:$domain;
    $this->connect($domain);
	$result=$this->connection->query($sql="SELECT * FROM `$table` WHERE id=0");
	if($this->connection->errno)
	throw new A_DBExceptionQuery($sql,$this->connection->errno,$this->connection->error);
    $finfo=$result->fetch_fields();
	foreach($finfo as $i=>$info)
	$fields[$i]=$info->name;
	$result->free();
	return $fields;
  }

/**
 * Возвращает список индексов в таблице БД.
 *
 * @param string $table Таблица БД.
 * @return array Список индексов в заданной таблице.
 */

  function getIndex($table,$domain='')
  {
    $domain=empty($domain)?A::$DOMAIN:$domain;
	$this->connect($domain);
    $index=array();
    $result=$this->connection->query($sql="SHOW KEYS FROM `$table`");
    if($this->connection->errno)
	throw new A_DBExceptionQuery($sql,$this->connection->errno,$this->connection->error);
    while($row=$result->fetch_assoc())
	$index[]=$row['Key_name'];
	$result->free();
	return $index;
  }

/**
 * Выполняет последовательность SQL запросов.
 * Работает в обход системы кэширования.
 *
 * @param string $sql Текст SQL скрипта.
 * @param string $domain Идентификатор сайта, если не указано то текущий.
 */

  function execSQL($sql,$domain='')
  {
    $domain=empty($domain)?A::$DOMAIN:$domain;
    $this->connect($domain);
	$sql=mb_convert_encoding($sql,"Windows-1251","UTF-8");
	$sql=rtrim($sql,"\n\r");
    $sql_len=strlen($sql);
    $char='';
    $string_start='';
    $in_string=false;
    $nothing=true;
    $time0=time();
    for($i=0;$i<$sql_len;++$i)
	{ $char = $sql[$i];
      if($in_string)
	  { while(1)
		{ $i = strpos($sql, $string_start, $i);
          if(!$i)
		  { $this->connection->query($_sql=mb_convert_encoding($sql,"UTF-8","Windows-1251"));
			if($this->connection->errno)
	        throw new A_DBExceptionQuery($_sql,$this->connection->errno,$this->connection->error);
		  }
          elseif($string_start == '`' || $sql[$i-1] != '\\')
		  { $string_start = '';
            $in_string = false;
            break;
          }
          else
		  { $j = 2;
            $escaped_backslash = false;
            while($i-$j > 0 && $sql[$i-$j] == '\\')
			{ $escaped_backslash = !$escaped_backslash;
              $j++;
            }
            if($escaped_backslash)
			{ $string_start  = '';
              $in_string = false;
              break;
            }
            else
			$i++;
          }
        }
      }
      elseif(($char == '-' && $sql_len > $i + 2 && $sql[$i + 1] == '-' && $sql[$i + 2] <= ' ') || $char == '#' || ($char == '/' && $sql_len > $i + 1 && $sql[$i + 1] == '*'))
	  { $i = strpos($sql, $char == '/' ? '*/' : "\n", $i);
        if($i === false) break;
        if($char == '/') $i++;
      }
	  elseif($char == ';')
	  { $this->connection->query($_sql=mb_convert_encoding(substr($sql, 0, $i),"UTF-8","Windows-1251"));
		if($this->connection->errno)
	    throw new A_DBExceptionQuery($_sql,$this->connection->errno,$this->connection->error);
        $nothing = true;
        $sql = ltrim(substr($sql, min($i + 1, $sql_len)));
        $sql_len = strlen($sql);
        if ($sql_len)
		$i = -1;
        else
		return;
      }
      elseif(($char == '"') || ($char == '\'') || ($char == '`'))
	  { $in_string = true;
        $nothing = false;
        $string_start = $char;
      }
      elseif ($nothing)
	  $nothing = false;
    }
    if(!empty($sql) && preg_match('/@[^[:space:]]+@/',$sql))
    { $this->connection->query($_sql=mb_convert_encoding($sql,"UTF-8","Windows-1251"));
      if($this->connection->errno)
	  throw new A_DBExceptionQuery($_sql,$this->connection->errno,$this->connection->error);
    }
  }

/**
 * Выполняет последовательность SQL запросов из файла.
 * Работает в обход системы кэширования.
 *
 * @param string $file Файл содержащий SQL скрипт.
 * @param string $domain Идентификатор сайта, если не указано то текущий.
 */

  function execSQLFile($file,$domain='')
  {
    $domain=empty($domain)?A::$DOMAIN:$domain;
    if($sql=@file_get_contents($file))
    $this->execSQL($sql,$domain);
  }
}