<?php
/** \file system/framework/fcache.php
 * Система кэширования в файлах.
 */
/**
 * @project Astra.CMS
 * @link http://a-cms.ru/
 * @copyright 2011 "Астра Вебтехнологии"
 * @author Vitaly Hohlov <admin@a-cms.ru>
 * @package AFramework
 */
/**************************************************************************/

/**
 * Класс реализует механизмы кэширования запросов к БД в файлах.
 */

class A_Cache_DB_File
{
  private $tableslog=array();

/**
 * Конструктор.
 */

  function __construct()
  {
    A::$DB->caching=true;
    A::$DB->cache = $this;
  }

/**
 * Очистка лога затронутых таблиц БД.
 */

  function clearTablesLog()
  {
    $this->tableslog=array();
  }

/**
 * Получение лога затронутых таблиц БД.
 *
 * @return array Список затронутых таблиц.
 */

  function getTablesLog()
  {
	return array_keys($this->tableslog);
  }

/**
 * Сохранение данных в кэш.
 *
 * @param string $sql Строка SQL запроса.
 * @param mixed $data Любого формата данные.
 * @param string $format Идентификатор формата данных.
 * @return array Сохраненные данные.
 */

  function set($sql,$data,$format='all')
  {
	if(preg_match("/RAND\(\)/i",$sql)) return false;

	$sql=str_replace('`','',$sql);
	$key=md5($sql.$format);
	$data=array('data'=>$data);

	if($tables=A_DB::getUseTables($sql))
	foreach($tables as $table)
	{ if(preg_match("/^([a-zA-Z0-9]+)_([a-zA-Z0-9_-]+)/i",$table,$matches))
	  { mk_dir("cache/db/{$matches[1]}");
	    mk_dir("cache/db/{$matches[1]}/{$matches[2]}");
	    $path="cache/db/{$matches[1]}/{$matches[2]}/{$key}.dat";
	  }
	  else
	  { mk_dir("cache/db/{$table}");
	    $path="cache/db/{$table}/{$key}.dat";
	  }
	  file_put_contents($path,serialize($data),LOCK_EX);
	}

	return $data['data'];
  }

/**
 * Извлечение данных из кэша.
 *
 * @param string $sql Строка SQL запроса.
 * @param string $format Идентификатор формата данных.
 * @return array Извлеченные данные или false.
 */

  function get($sql,$format='all')
  {
    if(preg_match("/RAND\(\)/i",$sql)) return false;

    $sql=str_replace('`','',$sql);
	$key=md5($sql.$format);
	$data=false;

    if($tables=A_DB::getUseTables($sql))
	foreach($tables as $table)
	{ if(preg_match("/^([a-zA-Z0-9]+)_([a-zA-Z0-9_-]+)/i",$table,$matches))
	  { $path="cache/db/{$matches[1]}/{$matches[2]}/{$key}.dat";
	    $this->tableslog[$table]=1;
	  }
	  else
	  $path="cache/db/{$table}/{$key}.dat";
	  if(!is_file($path))
	  return false;
	  if(!$data)
	  $data=unserialize(file_get_contents($path));
	}

	return $data;
  }

/**
 * Сброс кэша связанного с таблицей БД.
 *
 * @param string $table Таблица БД.
 */

  function resetTable($table)
  {
    if(preg_match("/^([a-zA-Z0-9]+)_([a-zA-Z0-9_-]+)/i",$table,$matches))
	$path="cache/db/{$matches[1]}/{$matches[2]}";
	else
	$path="cache/db/{$table}";
	delDir($path);
  }

/**
 * Сброс кэша связанного с разделом.
 *
 * @param string $section Полный строковой идентификатор раздела.
 */

  function resetSection($section)
  {
	if(preg_match("/^([a-zA-Z0-9]+)_([a-zA-Z0-9_-]+)/i",$section,$matches))
    { $domain=$matches[1];
	  $dir=dir("cache/db/$domain");
      while($file=$dir->read())
	  if($file!='.' && $file!='..' && is_dir("cache/db/$domain/$file") && preg_match("/^".$matches[2]."/i",$file))
	  delDir("cache/db/$domain/$file");
      $dir->close();
    }
  }

/**
 * Сброс кэша связанного с сайтом.
 *
 * @param string $domain Идентификатор сайта.
 */

  function resetDomain($domain)
  {
	delDir("cache/db/$domain");
  }
}

/**
 * Класс реализует механизмы кэширования страниц в файлах.
 */

class A_Cache_Page_File
{

/**
 * Восстановление объекта модуля и вывод страницы.
 */

  function restore()
  {
    $path='cache/pages/'.DOMAIN.'/'.LANG.'_'.SNAME;

	if(A::$URIPARAMS)
	$path.='/'.implode('/',A::$URIPARAMS);

	if($data=@file_get_contents($path.'/params.dat'));
    {
      $params=unserialize($data);

	  if(!empty($params['db']))
	  foreach($params['db'] as $table=>$time)
	  { if(preg_match("/^([a-zA-Z0-9]+)_([a-zA-Z0-9_-]+)/i",$table,$matches))
	    $tpath="cache/db/{$matches[1]}/{$matches[2]}";
		else
	    $tpath="cache/db/{$table}";
	    if(!is_dir($tpath) || filemtime($tpath)>$time)
	    return;
	  }

	  $values=array();

	  if(!empty($params['get']))
	  foreach($params['get'] as $param)
	  if(!empty($_GET[$param]))
	  $values['get'][$param]=$_GET[$param];

	  if(!empty($params['session']))
	  foreach($params['session'] as $param)
	  if(A_Session::is_set($param))
	  $values['session'][$param]=A_Session::get($param);

	  $key=md5(serialize($values));

	  if($data=@file_get_contents($path."/{$key}.dat"))
	  {
	    if(!empty($params['blocks']))
		foreach($params['blocks'] as $param)
		if(file_exists("blocks/{$param}/{$param}.php"))
		require_once("blocks/{$param}/{$param}.php");

		A::$MAINFRAME = unserialize($data);

		if(!empty($_REQUEST['action']))
        { switch($_REQUEST['action'])
	      { case "login": A::$AUTH->Login(); break;
	        case "logout": A::$AUTH->Logout(); break;
	      }
	      A::$OBSERVER->Event('Action',$_REQUEST['action']);
	      A::$MAINFRAME->Action($_REQUEST['action']);
        }

		if(!getAccess('read'))
        A::$MAINFRAME->goAccessDenied();

		A::$MAINFRAME->_display();
      }
    }
  }

/**
 * Сохранение текущего объекта модуля в кэш.
 */

  function save()
  {
	if(A::$MAINFRAME->cache_params)
    {
      mk_dir('cache/pages/'.DOMAIN);
	  mk_dir($path='cache/pages/'.DOMAIN.'/'.LANG.'_'.SNAME);

	  foreach(A::$URIPARAMS as $folder)
	  { $path.='/'.$folder;
	    mk_dir($path);
	  }

	  $params=A::$MAINFRAME->cache_params;
	  $params['db']=array();
	  $params['blocks']=array_keys(A::$MAINFRAME->useblocks);

	  if($tables=A::$CACHE->db->getTablesLog())
	  foreach($tables as $table)
	  { if(preg_match("/^([a-zA-Z0-9]+)_([a-zA-Z0-9_-]+)/i",$table,$matches))
	    $tpath="cache/db/{$matches[1]}/{$matches[2]}";
	    else
	    $tpath="cache/db/{$table}";
	    $params['db'][$table]=is_dir($tpath)?filemtime($tpath):0;
	  }

	  file_put_contents($path.'/params.dat',serialize($params),LOCK_EX);

	  $values=array();

	  if(!empty($params['get']))
	  foreach($params['get'] as $param)
	  if(!empty($_GET[$param]))
	  $values['get'][$param]=$_GET[$param];

	  if(!empty($params['session']))
	  foreach($params['session'] as $param)
	  if(A_Session::is_set($param))
	  $values['session'][$param]=A_Session::get($param);

	  $key=md5(serialize($values));

	  file_put_contents($path."/{$key}.dat",serialize(A::$MAINFRAME),LOCK_EX);
    }
  }
}

/**
 * Класс реализует механизмы кэширования блоков в файлах.
 */

class A_Cache_Block_File
{
/**
 * Восстановление данных у блока.
 *
 * @param object Объект блока.
 * @return boolean Успешность восстановления.
 */

  function restore($block)
  {
	$path='cache/vblocks/'.DOMAIN;
	$key=md5($block->block.serialize($block->params));

	if($data=@file_get_contents($path.'/'.$key.'.dat'))
    {
      $params=unserialize($data);

	  if(!empty($params['db']))
	  foreach($params['db'] as $table=>$time)
	  { if(preg_match("/^([a-zA-Z0-9]+)_([a-zA-Z0-9_-]+)/i",$table,$matches))
	    $tpath="cache/db/{$matches[1]}/{$matches[2]}";
		else
	    $tpath="cache/db/{$table}";
	    if(!is_dir($tpath) || filemtime($tpath)>$time)
	    return;
	  }

	  if(!empty($params['uri']))
	  { $path.='/'.LANG.'_'.SNAME;
	    if(A::$URIPARAMS)
	    $path.='/'.implode('/',A::$URIPARAMS);
	  }

	  $values=array();

	  if(!empty($params['get']))
	  foreach($params['get'] as $param)
	  if(!empty($_GET[$param]))
	  $values['get'][$param]=$_GET[$param];

	  if(!empty($params['session']))
	  foreach($params['session'] as $param)
	  if(A_Session::is_set($param))
	  $values['session'][$param]=A_Session::get($param);

	  $key=md5($key.serialize($values));

	  if($data=@file_get_contents($path."/{$key}.dat"))
	  { $block->_tpl_vars = unserialize($data);
	    return true;
	  }
    }
    return false;
  }

/**
 * Сохранение данных блока.
 *
 * @param object Объект блока.
 */

  function save($block)
  {
	if($block->cache_params)
    {
	  mk_dir($path='cache/vblocks/'.DOMAIN);

	  $key=md5($block->block.serialize($block->params));

      $params=$block->cache_params;
	  $params['db']=array();

      if($tables=A::$CACHE->db->getTablesLog())
	  foreach($tables as $table)
	  { if(preg_match("/^([a-zA-Z0-9]+)_([a-zA-Z0-9_-]+)/i",$table,$matches))
	    $tpath="cache/db/{$matches[1]}/{$matches[2]}";
	    else
	    $tpath="cache/db/{$table}";
	    $params['db'][$table]=is_dir($tpath)?filemtime($tpath):0;
	  }

	  file_put_contents($path.'/'.$key.'.dat',serialize($params),LOCK_EX);

	  if(!empty($params['uri']))
	  { mk_dir($path.='/'.LANG.'_'.SNAME);
	    foreach(A::$URIPARAMS as $folder)
	    { $path.='/'.$folder;
	      mk_dir($path);
	    }
	  }

	  $values=array();

	  if(!empty($params['get']))
	  foreach($params['get'] as $param)
	  if(!empty($_GET[$param]))
	  $values['get'][$param]=$_GET[$param];

	  if(!empty($params['session']))
	  foreach($params['session'] as $param)
	  if(A_Session::is_set($param))
	  $values['session'][$param]=A_Session::get($param);

	  $key=md5($key.serialize($values));

	  file_put_contents($path."/{$key}.dat",serialize($block->_tpl_vars),LOCK_EX);
    }
  }
}