<?php
/** \file system/framework/mcache.php
 * Система кэширования в memcahe.
 */
/**
 * @project Astra.CMS
 * @link http://a-cms.ru/
 * @copyright 2011 "Астра Вебтехнологии"
 * @author Vitaly Hohlov <admin@a-cms.ru>
 * @package AFramework
 */
/**************************************************************************/

/**
 * Класс реализует механизмы кэширования с использованием тегов.
 */

class A_MemCache extends MemCache
{
  private static $instance;

/**
 * Флаг успешного подключения к серверу memcache.
 */

  public $connected;

/**
 * Конструктор.
 */

  function __construct()
  {
	if(!empty($GLOBALS['A_MEMCACHED']))
	$this->connected=@parent::connect($GLOBALS['A_MEMCACHED']['host'],$GLOBALS['A_MEMCACHED']['port']);
	else
	$this->connected=@parent::connect('localhost',11211);

	if($this->connected)
	parent::setCompressThreshold(20000, 0.2);

	if(!empty($GLOBALS['A_MEMCACHED_FLUSH']))
	parent::flush();
  }

/**
 * Возвращает одиночный объект класса.
 */

  function getInstance()
  {
    if(!self::$instance)
    self::$instance = new A_MemCache;
    return self::$instance;
  }

/**
 * Сохранение данных в кэш.
 *
 * @param string $key Уникальный ключ.
 * @param mixed $data Данные.
 * @param array $tags=array() Список тегов.
 * @param integer $lifetime=0 Срок жизни.
 * @return mixed Сохраненные данные.
 */

  function setByTags($key,$data,$tags=array(),$lifetime=0)
  {
	$data=array('tags'=>array(),'data'=>$data);

	foreach($tags as $tag)
	{ $vtag=parent::get($tkey=md5($GLOBALS['A_DBCONFIG']['name'].$tag));
	  if(!$vtag)
	  parent::set($tkey,$vtag=time(),false,0);
	  $data['tags'][$tag]=$vtag;
	}

	return parent::set($key,$data,false,$lifetime);
  }

/**
 * Восстановление данных из кэша.
 *
 * @param string $key Уникальный ключ.
 * @return mixed Восстановленные данные или false.
 */

  function getByTags($key)
  {
	if($data=parent::get($key))
	{
	  foreach($data['tags'] as $tag=>$version)
	  { $vtag=parent::get(md5($GLOBALS['A_DBCONFIG']['name'].$tag));
        if(!$vtag || $vtag>$version)
	    { parent::delete($key,0);
	      return false;
	    }
	  }

	  return $data['data'];
	}
	return false;
  }

/**
 * Сбрасывает все элементы с заданным тегом.
 *
 * @param string $tag Тег.
 */

  function resetTag($tag)
  {
	if($vtag=parent::get($tkey=md5($GLOBALS['A_DBCONFIG']['name'].$tag)))
	parent::increment($tkey);
  }
}

/**
 * Класс реализует механизмы кэширования запросов к БД в memcache.
 */

class A_Cache_DB_Mem
{

  private $mcache;
  private $tableslog=array();

/**
 * Конструктор.
 */

  function __construct()
  {
    $this->mcache = A_MemCache::getInstance();

	if($this->mcache->connected)
	{ A::$DB->caching=true;
      A::$DB->cache = $this;
	}
	else
	{ A::$DB->caching=false;
	  A::$DB->__execute("UPDATE _options SET value=0 WHERE var='caching'");
	  A::$OPTIONS['caching']=0;
      delfile('cache/db/options.dat');
	}
  }

/**
 * Очистка лога затронутых таблиц БД.
 */

  function clearTablesLog()
  {
    $this->tableslog=array();
  }

/**
 * Получение лога затронутых таблиц БД.
 *
 * @return array Список затронутых таблиц.
 */

  function getTablesLog()
  {
    return array_keys($this->tableslog);
  }

/**
 * Сохранение данных в кэш.
 *
 * @param string $sql Строка SQL запроса.
 * @param mixed $data Любого формата данные.
 * @param string $format Идентификатор формата данных.
 * @return array Сохраненные данные.
 */

  function set($sql,$data,$format='all')
  {
	if(preg_match("/RAND\(\)/i",$sql)) return false;

	$sql=str_replace('`','',$sql);
	$key=md5($GLOBALS['A_DBCONFIG']['name'].$sql.$format);

	if($tables=A_DB::getUseTables($sql))
	return $this->mcache->setByTags($key,array('data'=>$data),$tables,3600)?$data:false;
	else
	return false;
  }

/**
 * Извлечение данных из кэша.
 *
 * @param string $sql Строка SQL запроса.
 * @param string $format Идентификатор формата данных.
 * @return array Извлеченные данные или false.
 */

  function get($sql,$format='all')
  {
    if(preg_match("/RAND\(\)/i",$sql)) return false;

    $sql=str_replace('`','',$sql);
	$key=md5($GLOBALS['A_DBCONFIG']['name'].$sql.$format);

	if($tables=A_DB::getUseTables($sql))
	foreach($tables as $table)
	$this->tableslog[$table]=1;

	return $this->mcache->getByTags($key);
  }

/**
 * Сброс кэша связанного с таблицей БД.
 *
 * @param string $table Таблица БД.
 */

  function resetTable($table)
  {
	$this->mcache->resetTag($table);
  }

/**
 * Сброс кэша связанного с разделом.
 *
 * @param string $section Полный строковой идентификатор раздела.
 */

  function resetSection($section)
  {
	$tables=A::$DB->getTables(getDomain($section));
	foreach($tables as $table)
	if(preg_match("/^".$section."/i",$table))
	$this->mcache->resetTag($table);
  }

/**
 * Сброс кэша связанного с сайтом.
 *
 * @param string $domain Идентификатор сайта.
 */

  function resetDomain($domain)
  {
	$tables=A::$DB->getTables($domain);
	foreach($tables as $table)
	if(preg_match("/^".$domain."_/i",$table))
	$this->mcache->resetTag($table);
  }
}

/**
 * Класс реализует механизмы кэширования страниц в memcache.
 */

class A_Cache_Page_Mem
{
  private $mcache;

/**
 * Конструктор.
 */

  function __construct()
  {
    $this->mcache = A_MemCache::getInstance();
  }

/**
 * Восстановление объекта модуля и вывод страницы.
 */

  function restore()
  {
	$key=md5(SECTION.':'.implode('/',A::$URIPARAMS));

	if($params=$this->mcache->get($key))
    {
	  $values=array();

	  if(!empty($params['get']))
	  foreach($params['get'] as $param)
	  if(!empty($_GET[$param]))
	  $values['get'][$param]=$_GET[$param];

	  if(!empty($params['session']))
	  foreach($params['session'] as $param)
	  if(A_Session::is_set($param))
	  $values['session'][$param]=A_Session::get($param);

	  $key=md5($key.serialize($values));

	  if($data=$this->mcache->getByTags($key))
	  {
		if(!empty($params['blocks']))
		foreach($params['blocks'] as $param)
		if(file_exists("blocks/{$param}/{$param}.php"))
		require_once("blocks/{$param}/{$param}.php");

		A::$MAINFRAME = unserialize($data);

		if(!empty($_REQUEST['action']))
        { switch($_REQUEST['action'])
	      { case "login": A::$AUTH->Login(); break;
	        case "logout": A::$AUTH->Logout(); break;
	      }
	      A::$OBSERVER->Event('Action',$_REQUEST['action']);
	      A::$MAINFRAME->Action($_REQUEST['action']);
        }

        if(!getAccess('read'))
        A::$MAINFRAME->goAccessDenied();

		A::$MAINFRAME->_display();
      }
    }
  }

/**
 * Сохранение текущего объекта модуля в кэш.
 */

  function save()
  {
	if(A::$MAINFRAME->cache_params)
    {
	  $key=md5(SECTION.':'.implode('/',A::$URIPARAMS));

	  $params=A::$MAINFRAME->cache_params;
	  $params['blocks']=array_keys(A::$MAINFRAME->useblocks);

	  $this->mcache->set($key,$params,false,3600);

	  $values=array();

	  if(!empty($params['get']))
	  foreach($params['get'] as $param)
	  if(!empty($_GET[$param]))
	  $values['get'][$param]=$_GET[$param];

	  if(!empty($params['session']))
	  foreach($params['session'] as $param)
	  if(A_Session::is_set($param))
	  $values['session'][$param]=A_Session::get($param);

	  $key=md5($key.serialize($values));

	  $this->mcache->setByTags($key,serialize(A::$MAINFRAME),A::$CACHE->db->getTablesLog(),1800);
    }
  }
}

/**
 * Класс реализует механизмы кэширования блоков в memcache.
 */

class A_Cache_Block_Mem
{
  private $mcache;

/**
 * Конструктор.
 */

  function __construct()
  {
    $this->mcache = A_MemCache::getInstance();
  }

/**
 * Восстановление данных у блока.
 *
 * @param object Объект блока.
 * @return boolean Успешность восстановления.
 */

  function restore($block)
  {
	$key=md5($GLOBALS['A_DBCONFIG']['name'].':'.$block->block.serialize($block->params));

	if($params=$this->mcache->get($key))
    {
	  $values=array('params'=>$block->params);

	  if(!empty($params['uri']))
	  $values['uri']=SECTION.':'.implode('/',A::$URIPARAMS);

	  if(!empty($params['get']))
	  foreach($params['get'] as $param)
	  if(!empty($_GET[$param]))
	  $values['get'][$param]=$_GET[$param];

	  if(!empty($params['session']))
	  foreach($params['session'] as $param)
	  if(A_Session::is_set($param))
	  $values['session'][$param]=A_Session::get($param);

	  $key=md5($key.serialize($values));

	  if($data=$this->mcache->getByTags($key))
	  { $block->_tpl_vars = $data;
	    return true;
	  }
    }
    return false;
  }

/**
 * Сохранение данных блока.
 *
 * @param object Объект блока.
 */

  function save($block)
  {
	if($block->cache_params)
    {
	  $key=md5($GLOBALS['A_DBCONFIG']['name'].':'.$block->block.serialize($block->params));

      $params=$block->cache_params;

      $this->mcache->set($key,$params,false,3600);

	  $values=array('params'=>$block->params);

	  if(!empty($params['uri']))
	  $values['uri']=SECTION.':'.implode('/',A::$URIPARAMS);

	  if(!empty($params['get']))
	  foreach($params['get'] as $param)
	  if(!empty($_GET[$param]))
	  $values['get'][$param]=$_GET[$param];

	  if(!empty($params['session']))
	  foreach($params['session'] as $param)
	  if(A_Session::is_set($param))
	  $values['session'][$param]=A_Session::get($param);

	  $key=md5($key.serialize($values));

	  $this->mcache->setByTags($key,$block->_tpl_vars,A::$CACHE->db->getTablesLog(),1800);
    }
  }
}